@@ -38,7 +38,7 @@
 #'
 #' @template param_allow_quantile_crossing
 #'
-#' @param init_weights Matrix of dimension 1xK or PxK used as starting weights. 1xK represents the constant solution with equal weights over all P, whereas specifying a PxK matrix allows different starting weights for each P.
+#' @param init A named list containing "init_weights": Matrix of dimension 1xK or PxK used as starting weights. 1xK represents the constant solution with equal weights over all P, whereas specifying a PxK matrix allows different starting weights for each P. "R0" a matrix of dimension PxK or 1xK used as starting regret.
 #' @param loss User specified loss array. Can also be a list with elements "loss_array"
 #' and "share", share mixes the provided loss with the loss calculated by
 #' profoc. 1 means, only the provided loss will be used. share can also be
@@ -100,34 +100,29 @@ online <- function(y, experts,
                    fixed_share = 0,
                    p_smooth_lambda = -Inf,
                    p_smooth_knot_distance = basis_knot_distance,
-                   p_smooth_knot_distance_power = basis_knot_distance_power, p_smooth_deg = basis_deg,
+                   p_smooth_knot_distance_power = basis_knot_distance_power,
+                   p_smooth_deg = basis_deg,
                    p_smooth_ndiff = 1.5,
                    gamma = 1,
                    parametergrid_max_combinations = 100,
                    parametergrid = NULL,
                    forget_past_performance = 0,
                    allow_quantile_crossing = FALSE,
-                   init_weights = NULL,
+                   init = NULL,
                    loss = NULL,
                    regret = NULL,
                    trace = TRUE) {
-
-    # Ensure that online_rcpp does not expand a grid for basis_knot_distance
-    # and p_smooth_knot_distance etc.
-    if (missing(p_smooth_knot_distance)) {
-        p_smooth_knot_distance <- as.numeric(c())
+    if (ncol(y) > 1 & !allow_quantile_crossing) {
+        warning("Warning: allow_quantile_crossing set to true since multivariate prediction target was provided.")
+        # Bool is set inside C++
     }
 
-    if (missing(p_smooth_knot_distance_power)) {
-        p_smooth_knot_distance_power <- as.numeric(c())
+    if (nrow(experts) - nrow(y) < 0) {
+        stop("Number of provided expert predictions has to match or exceed observations.")
     }
 
-    if (missing(p_smooth_deg)) {
-        p_smooth_deg <- as.numeric(c())
-    }
-
-    if (is.null(parametergrid)) {
-        parametergrid <- matrix(ncol = 0, nrow = 0)
+    if (nrow(y) <= lead_time) {
+        stop("Number of expert predictions need to exceed lead_time.")
     }
 
     if (is.null(loss)) {
@@ -152,6 +147,116 @@ online <- function(y, experts,
         regret_share <- regret$share
     }
 
+    if (is.null(parametergrid)) {
+        if (missing(p_smooth_knot_distance)) {
+            p_smooth_knot_distance <- 0
+            inh_kstep <- TRUE
+        } else {
+            inh_kstep <- FALSE
+        }
+
+        if (missing(p_smooth_knot_distance_power)) {
+            p_smooth_knot_distance_power <- 0
+            inh_kstep_p <- TRUE
+        } else {
+            inh_kstep_p <- FALSE
+        }
+
+        if (missing(p_smooth_deg)) {
+            p_smooth_deg <- 0
+            inh_deg <- TRUE
+        } else {
+            inh_deg <- FALSE
+        }
+
+        grid <- expand.grid(
+            basis_knot_distance,
+            basis_knot_distance_power,
+            basis_deg,
+            forget_regret,
+            soft_threshold,
+            hard_threshold,
+            fixed_share,
+            p_smooth_lambda,
+            p_smooth_knot_distance,
+            p_smooth_knot_distance_power,
+            p_smooth_deg,
+            p_smooth_ndiff,
+            gamma,
+            loss_share,
+            regret_share
+        )
+
+        if (inh_kstep) {
+            grid[, 9] <- grid[, 1]
+        }
+
+        if (inh_kstep_p) {
+            grid[, 10] <- grid[, 2]
+        }
+
+        if (inh_deg) {
+            grid[, 11] <- grid[, 3]
+        }
+
+        parametergrid <- as.matrix(grid)
+    } else if (ncol(parametergrid) != 15) {
+        stop("Please provide a parametergrid with 15 columns.")
+    }
+
+    if (nrow(parametergrid) > parametergrid_max_combinations) {
+        warning(
+            paste(
+                "Warning: Too many parameter combinations possible.",
+                parametergrid_max_combinations,
+                "combinations were randomly sampled. Results may depend on sampling."
+            )
+        )
+        parametergrid <- parametergrid[sample(
+            x = 1:nrow(parametergrid),
+            size = parametergrid_max_combinations
+        ), ]
+    }
+
+    if (is.null(init$init_weights)) {
+        init$init_weights <- matrix(
+            1 / dim(experts)[[3]],
+            nrow = dim(experts)[[2]],
+            ncol = dim(experts)[[3]]
+        )
+    } else if (nrow(init$init_weights) == 1) {
+        init$init_weights <- matrix(init$init_weights,
+            nrow = dim(experts)[[2]],
+            ncol = dim(experts)[[3]],
+            byrow = TRUE
+        )
+    } else if (
+        (nrow(init$init_weights) != 1 &
+            nrow(init$init_weights) != dim(experts)[[2]]) |
+            ncol(init$init_weights) != dim(experts)[[3]]) {
+        stop("Either a 1xK or PxK matrix of initial weights must be supplied.")
+    }
+    init$init_weights <- pmax(init$init_weights, exp(-350))
+    init$init_weights <- init$init_weights / rowSums(init$init_weights)
+
+    if (is.null(init$R0)) {
+        init$R0 <- matrix(0,
+            nrow = dim(experts)[[2]],
+            ncol = dim(experts)[[3]],
+        )
+    } else if (nrow(init$R0) == 1) {
+        init$R0 <- matrix(init$R0,
+            nrow = dim(experts)[[2]],
+            ncol = dim(experts)[[3]],
+            byrow = TRUE
+        )
+    } else if (
+        (nrow(init$R0) != 1 &
+            nrow(init$R0) != dim(experts)[[2]]) |
+            ncol(init$R0) != dim(experts)[[3]]) {
+        stop("R0 must be 1xK or PxK.")
+    }
+
     model <- online_rcpp(
         y = y, experts = experts, tau = tau,
         lead_time = lead_time,
@@ -159,28 +264,13 @@ online <- function(y, experts,
         loss_parameter = loss_parameter,
         loss_gradient = loss_gradient,
         method = method,
-        basis_knot_distance = basis_knot_distance,
-        basis_knot_distance_power = basis_knot_distance_power,
-        basis_deg = basis_deg,
-        forget_regret = forget_regret,
-        soft_threshold = soft_threshold,
-        hard_threshold = hard_threshold,
-        fixed_share = fixed_share,
-        p_smooth_lambda = p_smooth_lambda,
-        p_smooth_knot_distance = p_smooth_knot_distance,
-        p_smooth_knot_distance_power = p_smooth_knot_distance_power,
-        p_smooth_deg = p_smooth_deg,
-        p_smooth_ndiff = p_smooth_ndiff,
-        gamma = gamma,
-        parametergrid_max_combinations = parametergrid_max_combinations,
-        parametergrid = parametergrid,
+        param_grid = parametergrid,
         forget_past_performance = forget_past_performance,
         allow_quantile_crossing = allow_quantile_crossing,
-        init_weights = init_weights,
+        w0 = init$init_weights,
+        R0 = init$R0,
         loss_array = loss_array,
-        loss_share = loss_share,
         regret_array = regret_array,
-        regret_share = regret_share,
         trace = trace
     )
 
